@@ -27,8 +27,6 @@
 #include <signal.h>
 #include <errno.h>
 
-#include "gtkemuvt102.h"
-
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
@@ -207,6 +205,7 @@ static void
 gtk_tty_init (GtkTty *tty)
 {
   tty->key_states = GTK_TTY_STATE_NONE;
+
   tty->ignore_scroll_lock = FALSE;
   tty->ignore_num_lock = FALSE;
   tty->ignore_caps_lock = FALSE;
@@ -217,6 +216,12 @@ gtk_tty_init (GtkTty *tty)
   tty->key_pad_enter = FALSE;
   tty->key_pad_char = 0;
   
+  tty->pty_fd = -1;
+  tty->tty_name = NULL;
+  tty->input_tag = 0;
+  
+  tty->vtemu = NULL;
+
   tty->pid = 0;
   tty->prg_name = NULL;
   tty->exit_status = 0;
@@ -225,21 +230,6 @@ gtk_tty_init (GtkTty *tty)
   tty->sys_sec = 0;
   tty->user_usec = 0;
   tty->user_sec = 0;
-  
-  tty->pty_fd = -1;
-  tty->tty_name = NULL;
-  tty->input_tag = 0;
-  
-  tty->emu_data = NULL;
-  tty->emu_term = NULL;
-  tty->emu_func = NULL;
-  tty->emu_reset_func = NULL;
-  tty->emu_free_func = NULL;
-  gtk_tty_set_emu (tty,
-		   gtk_emu_vt102_new (),
-		   gtk_emu_vt102,
-		   gtk_emu_vt102_reset,
-		   gtk_emu_vt102_free);
 }
 
 
@@ -249,16 +239,15 @@ gtk_tty_new (guint width,
 	     guint scrollback)
 {
   GtkTty *tty;
+  GList	 *list;
   
   tty = gtk_type_new (gtk_tty_get_type ());
   scrollback = MIN (scrollback, MAX_SCROLL_BACK);
   gtk_term_setup (GTK_TERM (tty), width, height, width, scrollback);
-  
-  if (tty->emu_reset_func)
-  {
-    g_free (tty->emu_term);
-    tty->emu_term = g_strdup ((* tty->emu_reset_func) (tty, tty->emu_data, TRUE));
-  }
+
+  list = gtk_vtemu_get_types ();
+  tty->vtemu = gtk_vtemu_new (GTK_TERM (tty), list->data);
+  g_list_free (list);
   
   return GTK_WIDGET (tty);
 }
@@ -730,8 +719,9 @@ gtk_tty_destroy (GtkObject *object)
   g_return_if_fail (GTK_IS_TTY (object));
   
   tty = GTK_TTY (object);
-  
-  gtk_tty_set_emu (tty, NULL, NULL, NULL, NULL);
+
+  if (tty->vtemu)
+    gtk_vtemu_destroy (tty->vtemu);
   
   gtk_tty_os_close_pty (tty);
   
@@ -900,23 +890,23 @@ gtk_tty_input_func (gpointer		data,
 
 void
 gtk_tty_put_out (GtkTty		*tty,
-		 guchar		*buffer,
+		 const guchar	*buffer,
 		 guint		count)
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
   
-  if (tty->emu_func)
+  if (tty->vtemu)
     while (count)
     {
-      count -= (* tty->emu_func) (tty, tty->emu_data, buffer, count);
+      count -= gtk_vtemu_input (tty->vtemu, buffer, count);
     }
 }
 
 void
-gtk_tty_put_in (GtkTty	       *tty,
-		guchar	       *buffer,
-		guint	       count)
+gtk_tty_put_in (GtkTty		*tty,
+		const guchar	*buffer,
+		guint		count)
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
@@ -928,28 +918,16 @@ gtk_tty_put_in (GtkTty	       *tty,
 }
 
 void
-gtk_tty_set_emu (GtkTty		 *tty,
-		 gpointer	 emu_data,
-		 GtkEmuFunc	 *emu_func,
-		 GtkEmuResetFunc *emu_reset_func,
-		 GtkEmuFreeFunc	 *emu_free_func)
+gtk_tty_change_emu (GtkTty	*tty,
+		    GtkVtEmu	*vtemu)
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
-  
-  if (tty->emu_data && tty->emu_free_func)
-    (*tty->emu_free_func) (tty->emu_data);
-  
-  if (tty->emu_term)
-  {
-    g_free (tty->emu_term);
-    tty->emu_term = NULL;
-  }
-  
-  tty->emu_data = emu_data;
-  tty->emu_func = emu_func;
-  tty->emu_reset_func = emu_reset_func;
-  tty->emu_free_func = emu_free_func;
+
+  if (tty->vtemu)
+    gtk_vtemu_destroy (tty->vtemu);
+
+  tty->vtemu = vtemu;
 }
 
 void
@@ -1016,11 +994,8 @@ gtk_tty_execute (GtkTty		*tty,
     tty->prg_name = g_strdup (prg_name);
   
   
-  if (tty->emu_reset_func)
-  {
-    g_free (tty->emu_term);
-    tty->emu_term = g_strdup ((* tty->emu_reset_func) (tty, tty->emu_data, FALSE));
-  }
+  if (tty->vtemu)
+    gtk_vtemu_reset (tty->vtemu, FALSE);
   
   if (!tty->tty_name)
   {
@@ -1057,16 +1032,16 @@ gtk_tty_execute (GtkTty		*tty,
     {
       gchar *empty_term = "TERM=";
 
-      if (!tty->emu_term)
+      if (!tty->vtemu)
 	putenv (empty_term);
       else
       {
 	gchar *buffer;
 	
-	buffer = g_new (gchar, strlen (empty_term) + strlen (tty->emu_term) + 1);
+	buffer = g_new (gchar, strlen (empty_term) + strlen (tty->vtemu->terminal_type) + 1);
 	buffer[0] = 0;
 	strcat (buffer, empty_term);
-	strcat (buffer, tty->emu_term);
+	strcat (buffer, tty->vtemu->terminal_type);
 
 	if (putenv (buffer) < 0)
 	{
