@@ -20,11 +20,19 @@
 #include	"gtktty.h"
 #include	"gtkled.h"
 #include	"config.h"
+#include	"gemvt.xpm"
+#include	"gem-r.xpm"
+#include	"gem-g.xpm"
+#include	"gem-b.xpm"
 #include	<gtk/gtk.h>
 #include	<signal.h>
 #include	<unistd.h>
 #include	<string.h>
 
+#ifdef	GNOMIFY
+#include	"gnome.h"
+#endif	GNOMIFY
+
 
 /* --- typedefs --- */
 typedef struct
@@ -66,6 +74,13 @@ static	void	gvt_term_set_color_table(GtkTerm	*term,
 					 GvtColorEntry	*color_table,
 					 guint		len,
 					 guint		first);
+static
+GtkWidget*	gvt_gem_create		(GtkContainer	*parent,
+					 gchar		 col);
+static	void	gvt_gem_set_color	(GtkWidget	*gem,
+					 gchar		 col);
+static	void	gvt_gem_rotate		(GtkWidget	*gem);
+
 
 
 /* --- external variables --- */
@@ -97,26 +112,70 @@ int
 main	(int	argc,
 	 char	*argv[])
 {
-  GtkWidget	*window;
-  GtkWidget	*main_vbox;
-  GtkWidget	*tty_vbox;
-  GtkWidget	*tty_hbox;
-  GtkWidget	*status_hbox;
-  GtkWidget	*led_hbox;
-  GtkWidget	*led[8];
-  GtkWidget	*label;
-  GtkWidget	*tty;
-  GtkWidget	*exec_hbox;
-  GtkWidget	*entry;
-  GtkWidget	*button;
-  guchar	*message;
-  guint		i;
+  register gchar	*home_dir;
+  register gchar	*rc_file;
+  register guchar	*string;
+  register GtkWidget	*window;
+  register GtkWidget	*main_vbox;
+  register GtkWidget	*tty_vbox;
+  register GtkWidget	*tty_hbox;
+  register GtkWidget	*status_hbox;
+  register GtkWidget	*led_hbox;
+  GtkWidget		*led[8];
+  register GtkWidget	*label;
+  register GtkWidget	*tty;
+  register GtkWidget	*exec_hbox;
+  register GtkWidget	*entry;
+  register GtkWidget	*button;
+  register GtkWidget	*gem;
+  register guint	i;
+  
 
   prg_name = g_strdup (argv[0]);
   
-  /* gtk/gdk initialization
+  /* Gtk+/GNOME initialization
    */
+#ifdef	GNOMIFY
+  gnome_init(&argc, &argv);
+#else	/* !GNOMIFY */
   gtk_init (&argc, &argv);
+#endif	/* !GNOMIFY */
+
+  /* parse ~/.gtkrc, (this file also ommitted by the gnome_init stuff
+   */
+  home_dir = getenv ("HOME");
+  if (!home_dir) /* fatal! */
+    home_dir = ".";
+  
+  string = "gtkrc";
+  rc_file = g_new (gchar, strlen (home_dir) + 2 + strlen (string) + 1);
+  *rc_file = 0;
+  strcat (rc_file, home_dir);
+  strcat (rc_file, "/.");
+  strcat (rc_file, string);
+
+  printf("%s\n", rc_file);
+  gtk_rc_parse (rc_file);
+  g_free (rc_file);
+  
+  string = strrchr (prg_name, '/');
+  if (string)
+    string++;
+  else
+    string = prg_name;
+  rc_file = g_new (gchar, strlen (home_dir) + 2 + strlen (string) + 2 + 1);
+  *rc_file = 0;
+  strcat (rc_file, home_dir);
+  strcat (rc_file, "/.");
+  strcat (rc_file, string);
+  strcat (rc_file, "rc");
+
+  /* parse ~/.<prgname>rc
+   */
+  printf("%s\n", rc_file);
+  /* FIXME: invoke rc-parser here, this requires GScanner in GLib first ;( */
+  g_free (rc_file);
+
   
   /* catch system signals
    */
@@ -170,12 +229,13 @@ main	(int	argc,
     gtk_widget_show (led[i]);
   }
   
-  message = "Hi, this is GemVt bothering you ;)\n\r"
-	    "Enter `-bash' or something the like and\n\r"
-	    "Have Fun!!!\n\r";
+  string =
+    "Hi, this is GemVt bothering you ;)\n\r"
+    "Enter `-bash' or something the like and\n\r"
+    "Have Fun!!!\n\r";
   
   tty = gtk_tty_new (80, 25, 99);
-  gtk_tty_put_out (GTK_TTY (tty), message, strlen (message));
+  gtk_tty_put_out (GTK_TTY (tty), string, strlen (string));
   gtk_box_pack_start (GTK_BOX (tty_vbox), tty, TRUE, TRUE, 5);
   for (i = 0; i < 8; i++)
   {
@@ -227,6 +287,16 @@ main	(int	argc,
 		      GTK_SIGNAL_FUNC (gvt_kill),
 		      tty);
 
+  button = gtk_button_new ();
+  gtk_box_pack_start (GTK_BOX (exec_hbox), button, FALSE, TRUE, 5);
+  gtk_widget_show (button);
+  gem = gvt_gem_create (GTK_CONTAINER (button), 'r');
+  gtk_container_add (GTK_CONTAINER (button), gem);
+  gtk_signal_connect_object (GTK_OBJECT (button),
+			     "clicked",
+			     GTK_SIGNAL_FUNC (gvt_gem_rotate),
+			     (GtkObject*) gem);
+
   button = gtk_button_new_with_label ("Close");
   gtk_box_pack_start (GTK_BOX (main_vbox), button, FALSE, TRUE, 5);
   gtk_widget_show (button);
@@ -399,15 +469,160 @@ gvt_tty_key_press (GtkTty         *tty,
   if (tty->pid != 0)
     return FALSE;
   else
-    {
-      /* we check the length of the original code to make sure
-       * we don't react on shift and stuff...
-       */
-      if (length > 0)
-	gtk_tty_put_in (tty, "*", 1);
+  {
+    /* we check the length of the original code to make sure
+     * we don't react on shift and stuff...
+     */
+    if (keyval < 127)
+      gtk_tty_put_in (tty, char_code, length);
+    
+    return TRUE;
+  }
+}
 
-      return TRUE;
-    }
+GtkWidget*
+gvt_gem_create (GtkContainer *parent,
+		gchar	     col)
+{
+  GtkWidget	*vbox;
+  GtkWidget	*pixmap;
+  GdkPixmap	*xpm_map;
+  GdkBitmap	*bit_map;
+
+  g_assert (parent != NULL);
+  g_assert (GTK_IS_CONTAINER (parent));
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_container_add (parent, vbox);
+  gtk_widget_show (vbox);
+  gtk_object_set_data (GTK_OBJECT (vbox), "active-gem", NULL);
+
+  bit_map = NULL;
+  xpm_map = gdk_pixmap_create_from_xpm_d (vbox->window,
+					  &bit_map,
+					  &gtk_widget_get_style (vbox)->bg[GTK_STATE_NORMAL],
+					  gem_r_xpm);
+  pixmap = gtk_pixmap_new (xpm_map, bit_map);
+  gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
+  gtk_object_set_data (GTK_OBJECT (pixmap), "gem-color", "red");
+  gtk_object_set_data (GTK_OBJECT (vbox), "red-gem", pixmap);
+
+  bit_map = NULL;
+  xpm_map = gdk_pixmap_create_from_xpm_d (vbox->window,
+					  &bit_map,
+					  &gtk_widget_get_style (vbox)->bg[GTK_STATE_NORMAL],
+					  gem_g_xpm);
+  pixmap = gtk_pixmap_new (xpm_map, bit_map);
+  gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
+  gtk_object_set_data (GTK_OBJECT (pixmap), "gem-color", "green");
+  gtk_object_set_data (GTK_OBJECT (vbox), "green-gem", pixmap);
+
+  bit_map = NULL;
+  xpm_map = gdk_pixmap_create_from_xpm_d (vbox->window,
+					  &bit_map,
+					  &gtk_widget_get_style (vbox)->bg[GTK_STATE_NORMAL],
+					  gem_b_xpm);
+  pixmap = gtk_pixmap_new (xpm_map, bit_map);
+  gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
+  gtk_object_set_data (GTK_OBJECT (pixmap), "gem-color", "blue");
+  gtk_object_set_data (GTK_OBJECT (vbox), "blue-gem", pixmap);
+
+  gvt_gem_set_color (vbox, col);
+
+  return vbox;
+}
+
+static void
+gvt_gem_set_color (GtkWidget      *gem,
+		   gchar	   col)
+{
+  GtkObject	*object;
+  GtkWidget	*pixmap;
+  GtkWidget	*new_pixmap;
+
+  g_assert (gem != NULL);
+  g_assert (GTK_IS_OBJECT (gem));
+
+  object = GTK_OBJECT (gem);
+
+  pixmap = gtk_object_get_data (object, "active-gem");
+
+  switch (col)
+  {
+  case  'r':
+    new_pixmap = gtk_object_get_data (object, "red-gem");
+    break;
+
+  case  'g':
+    new_pixmap = gtk_object_get_data (object, "green-gem");
+    break;
+
+  case  'b':
+    new_pixmap = gtk_object_get_data (object, "blue-gem");
+    break;
+
+  default:
+    new_pixmap = NULL;
+  }
+
+  if (pixmap)
+  {
+    gtk_container_block_resize (GTK_CONTAINER (pixmap->parent));
+    gtk_widget_hide (pixmap);
+  }
+
+  gtk_object_set_data (object, "active-gem", new_pixmap);
+  if (new_pixmap)
+    gtk_widget_show (new_pixmap);
+
+  if (pixmap)
+    gtk_container_unblock_resize (GTK_CONTAINER (pixmap->parent));
+}
+
+static void
+gvt_gem_rotate (GtkWidget      *gem)
+{
+  GtkObject	*object;
+  GtkWidget	*pixmap;
+  gchar		*old_col;
+  gchar		col;
+
+  g_assert (gem != NULL);
+  g_assert (GTK_IS_OBJECT (gem));
+
+  object = GTK_OBJECT (gem);
+
+  pixmap = gtk_object_get_data (object, "active-gem");
+
+  if (pixmap)
+  {
+    g_assert (GTK_IS_OBJECT (pixmap));
+
+    old_col = gtk_object_get_data (GTK_OBJECT (pixmap), "gem-color");
+  }
+  else
+    old_col = NULL;
+
+  if (!old_col)
+    old_col = "red";
+
+  switch (old_col[0])
+  {
+  default:
+  case  'r':
+    col = 'g';
+    break;
+
+  case  'g':
+    col = 'b';
+    break;
+
+  case  'b':
+    col = 'r';
+    break;
+  }
+
+  gvt_gem_set_color (gem, col);
 }
 
 static void
@@ -512,11 +727,18 @@ gvt_program_exit (GtkTty	 *tty,
   if (1)
   {
     gchar string[256];
+    gchar sys_time[64];
+    gchar user_time[64];
+    guint i;
 
-    sprintf (string,
-	     "System: %f  User: %f",
-	     tty->sys_sec + tty->sys_usec/1000000.0,
-	     tty->user_sec + tty->user_usec/1000000.0);
+    sprintf (sys_time, "%f", tty->sys_sec + tty->sys_usec/1000000.0);
+    for (i = strlen (sys_time) - 1; sys_time[i] == '0'; i--)
+      sys_time[i] = 0;
+    sprintf (user_time, "%f", tty->user_sec + tty->user_usec/1000000.0);
+    for (i = strlen (user_time) - 1; user_time[i] == '0'; i--)
+      user_time[i] = 0;
+
+    sprintf (string, "System: %s  User: %s", sys_time, user_time);
     gtk_label_set (GTK_LABEL (time_label), string);
   }
   
@@ -566,11 +788,11 @@ gvt_term_set_color_table (GtkTerm       *term,
     if (!color_table[i].fore_col)
     {
       color_table[i].fore_col = g_new0 (GdkColor, 1);
-      color_table[i].fore_col->red = color_table[i].back_val >> 8 & 0xff00;
+      color_table[i].fore_col->red = color_table[i].fore_val >> 8 & 0xff00;
       color_table[i].fore_col->red += color_table[i].fore_col->red >> 8;
-      color_table[i].fore_col->green = color_table[i].back_val & 0xff00;
+      color_table[i].fore_col->green = color_table[i].fore_val & 0xff00;
       color_table[i].fore_col->green += color_table[i].fore_col->green >> 8;
-      color_table[i].fore_col->blue = color_table[i].back_val & 0xff;
+      color_table[i].fore_col->blue = color_table[i].fore_val & 0xff;
       color_table[i].fore_col->blue += color_table[i].fore_col->blue << 8;
       
       if (!gdk_color_alloc (cmap, color_table[i].fore_col))
@@ -587,11 +809,11 @@ gvt_term_set_color_table (GtkTerm       *term,
     if (!color_table[i].dim_col)
     {
       color_table[i].dim_col = g_new0 (GdkColor, 1);
-      color_table[i].dim_col->red = color_table[i].back_val >> 8 & 0xff00;
+      color_table[i].dim_col->red = color_table[i].dim_val >> 8 & 0xff00;
       color_table[i].dim_col->red += color_table[i].dim_col->red >> 8;
-      color_table[i].dim_col->green = color_table[i].back_val & 0xff00;
+      color_table[i].dim_col->green = color_table[i].dim_val & 0xff00;
       color_table[i].dim_col->green += color_table[i].dim_col->green >> 8;
-      color_table[i].dim_col->blue = color_table[i].back_val & 0xff;
+      color_table[i].dim_col->blue = color_table[i].dim_val & 0xff;
       color_table[i].dim_col->blue += color_table[i].dim_col->blue << 8;
       
       if (!gdk_color_alloc (cmap, color_table[i].dim_col))
@@ -612,11 +834,11 @@ gvt_term_set_color_table (GtkTerm       *term,
     if (!color_table[i].bold_col)
     {
       color_table[i].bold_col = g_new0 (GdkColor, 1);
-      color_table[i].bold_col->red = color_table[i].back_val >> 8 & 0xff00;
+      color_table[i].bold_col->red = color_table[i].bold_val >> 8 & 0xff00;
       color_table[i].bold_col->red += color_table[i].bold_col->red >> 8;
-      color_table[i].bold_col->green = color_table[i].back_val & 0xff00;
+      color_table[i].bold_col->green = color_table[i].bold_val & 0xff00;
       color_table[i].bold_col->green += color_table[i].bold_col->green >> 8;
-      color_table[i].bold_col->blue = color_table[i].back_val & 0xff;
+      color_table[i].bold_col->blue = color_table[i].bold_val & 0xff;
       color_table[i].bold_col->blue += color_table[i].bold_col->blue << 8;
       
       if (!gdk_color_alloc (cmap, color_table[i].bold_col))
